import sys
sys.path.insert(0,'')
from pygame.locals import *
from OpenGL.GL import *
from OpenGL.GLU import *
import os
import random
import pygame

import mymap
import mytexture
import myfont
import myobjects
import mysmoke
import mymenu

class Game:
	def __init__( self ):
		self.map = None
		self.camx = 4
		self.camy = 6.20
		self.camz = 40

		#window size
		self.width = 1024	
		self.height = 768

		#animate variables
		self.ux = 0.5
		self.uy = 0.2
		self.TimerEvent = USEREVENT + 1		
		self.ground = None
		self.hud = None
		pygame.time.set_timer( self.TimerEvent, 60 );


		#hero object
		self.hero = myobjects.Object( .2 )
		self.follow = None
		self.collecting = False
		self.maxheat = 150
		self.heat = 0
		self.heattex = None
		
		#font object
		self.font = None

		#cop objects
		self.maxcops = 5
		self.cops = []
		self.coptimer = 0

		#wood and cops counter
		self.coptraps = 0
		self.woodcount = 50 

		#game related
		self.gameWon = False
		self.done = False

		self.maps= [ 'MAP1','MAP2','MAP3', 'MAP4', 'MAP5' ]
		self.level = 0
		self.started = False

		self.mapname = os.path.join( "maps",self.maps[0]+".txt")
		
		#show when you get +10 wood
		self.woodgrowup = []
		self.growuptex = None
		self.growdowntex = None

		#game menu
		self.menu = None

		#for showing +10 when collected
		self.woodtimer = 0
		
		#sounds related
		self.soundsEnabled = False
		self.sounds = {}


	def glInit( self, width, height ):
		glViewport( 0, 0, width, height )
		glMatrixMode( GL_PROJECTION )
		glLoadIdentity( )
		gluPerspective( 30, 1.33, 0.1, 400)
		glMatrixMode( GL_MODELVIEW )
		glLoadIdentity( )
		gluLookAt( self.camx, self.camy, self.camz, self.camx, 0, 20, 0, 1, 0 )

	def init( self ):
		#initialization for game
		self.map = mymap.Map( self.mapname )
		self.ground = mytexture.loadTexture( os.path.join("models","grass.tga") )
		self.hud = mytexture.loadTexture( os.path.join("models","hud.tga") )
		self.heattex = mytexture.loadTexture( os.path.join("models","heat.tga") )
		self.growuptex = mytexture.loadTexture( os.path.join("models","10.tga") )
		self.growdowntex = mytexture.loadTexture( os.path.join("models","20.tga") )
		self.font = myfont.myFont( os.path.join("models","Font.tga") )
		self.smoke = mysmoke.Smoke( 0, 0, 0, 0, 3, 0, 5, 0.4, os.path.join("models","whitesmoke.tga") );
		self.map.bottomtex = mytexture.loadTexture( os.path.join("models","treebottom.tga") )	
		self.map.traptex = mytexture.loadTexture( os.path.join("models","trap.tga") )
		self.hero.ac3d_obj = self.map.heromodel
		self.follow = self.hero
		(self.hero.x, self.hero.z ) = random.choice( self.map.spawn )

		#initialize menu
		self.menu = mymenu.MenuPage( self.font, 0, ["START", "EXIT"] )
		self.menu.model = mytexture.loadTexture( os.path.join("models","logo.tga") )

		#initialization of sounds
		try:
			pygame.mixer.music.load( os.path.join("music","Grunge.wav") )	
			#pygame.mixer.music.set_volume( .1 )
			self.sounds['police'] = pygame.mixer.Sound( os.path.join("sounds","police.wav") )
			self.sounds['chainsaw'] = pygame.mixer.Sound( os.path.join( "sounds", "chainsaw.wav") )
			self.sounds['alarm'] = pygame.mixer.Sound( os.path.join("sounds", "alarm.wav") )
			self.sounds['trap'] = pygame.mixer.Sound( os.path.join("sounds","collision.ogg") )
			self.soundsEnabled = True
#			pygame.mixer.music.play( )
		except:
			self.soundsEnabled = False;
		
		#some initialization for opengl
		glClearColor( 0.0, 0.0, 0.0, 0.0 )
		glEnable( GL_DEPTH_TEST )
		glDepthFunc( GL_LEQUAL )
		glAlphaFunc( GL_GREATER, 0 )
		glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA)
		self.glInit( self.width, self.height)

	def restart( self ):
		#map related stuff
		self.map.load(self.mapname)
		#self.follow = self.hero
		(self.hero.x, self.hero.z ) = random.choice( self.map.spawn )
		self.hero.direction = None
		self.hero.resetPrevData( )

		#animation related 
		self.collecting = False
		self.cops = []
		self.maxcops = self.map.maxcops
		self.coptimer = 0

		#cam movement related
		self.ux = 0.2
		self.uy = 0.1

		self.camx = self.hero.x
		self.camy = 6.2

		#game stats related
		self.coptraps = 0
		self.woodcount = 50
#		self.woodcount = int(self.map.wood/4.0)

		#game related vars
		self.done = False
		self.gameWon = False
		self.started = False
		self.hero.alive = True
		self.heat = 0
		self.maxheat = (self.level+1)*50
	
	def update( self ):		
		#update hero position
		self.hero.update( self.map )

		#check if we reached check point
		if self.hero.objid == mymap.TYPE_DESTINATION:
			if ( self.map.wood <= self.woodcount and self.map.cops <= self.coptraps ):
				self.gameWon = True
				self.hero.alive = False

		#update the cam x variable	
		if self.follow == None or self.started == False:
			x = self.camx + self.ux	
			y = self.camy + self.uy

			if y > 6 and y < 20 :
				self.camy = y
			else:
				self.uy = -1*self.uy

			if x > 0 and x < 59:
				self.camx = x
			else: 
				self.ux = -1*self.ux
		else:
			self.camx = self.follow.x
	
		y = self.camy + self.uy
		if y > 6 and y < 20 :
			self.camy = y

		self.smoke.animate(0.04)
	
		#user did not press any key
		if not(self.started):
			return

		#add cops using timer
		self.coptimer += 1
		if self.coptimer >= len(self.cops)*5+50:
			self.coptimer = 0
			if len(self.cops) < self.maxcops:
				cop = myobjects.Object( )
				(cop.x, cop.z) = random.choice( self.map.spawn )
				cop.ac3d_obj = self.map.copmodel
				cop.findDirection( (self.hero.x, self.hero.z), mymap.TYPE_TRAP )
				#set the spawn position 
				cop.spawn = [random.choice( self.map.spawn )]
				cop.setCopTimer()
				self.cops.append( cop )
		

		#wood timer (displays 10)
		self.woodtimer += 1
		if self.woodtimer >= 10:
			self.woodtimer = 0
			self.woodcount -= 1
			#using wood timer to increase heat of engine
			self.heat += 1
			if self.collecting:

				#play sounds
				self.sounds['chainsaw'].fadeout( 60 )
				self.sounds['chainsaw'].play()
				self.woodcount += 10
				sprite = mysmoke.elem()
				sprite.x = self.map.curtree[0]
				sprite.y = .5
				sprite.z = self.map.curtree[1]
				sprite.texid = self.growuptex
				self.woodgrowup.append( sprite)
				
			if self.hero.objid == mymap.TYPE_TRAP:
				self.woodcount -= 20
				sprite = mysmoke.elem()
				sprite.x = self.hero.x
				sprite.y = 0.5
				sprite.z = self.hero.z
				sprite.texid = self.growdowntex
				self.woodgrowup.append( sprite)

		#if the wood is over the game ends
		if self.woodcount < 0 or self.heat >= self.maxheat:
			self.gameWon = False
			self.hero.alive = False

		nlist = []
		police_siren= False
		for cop in self.cops:
			#check collision with map
			if cop.coptimer > 0:
				cop.findDirection( (self.hero.x, self.hero.z), mymap.TYPE_TRAP )
			else:
				spawnx = cop.spawn[0][0]
				spawny = cop.spawn[0][1]
				if spawnx -2 < cop.x and cop.x < spawnx+2:
					if spawny -2 < cop.z and cop.z < spawny+2:
						cop.setCopTimer()
				cop.findDirection( cop.spawn[0], mymap.TYPE_TRAP )
				
			cop.update( self.map )

			#check if the police is in the proximity
			if self.hero.x-5 < cop.x and self.hero.x+5 > cop.x :
				police_siren = True
				
			#check collision with hero
			if cop.x-1 <= self.hero.x and self.hero.x <= cop.x+1:
				if( cop.z-1 <= self.hero.z and self.hero.z <= cop.z+1 ):
					self.gameWon = False
					self.hero.alive = False
					#self.done = True

			if cop.alive:
				nlist.append(cop)
			else:
				self.coptraps+=1
				
		#if there is atleast one cop in view play siren
		if police_siren:
			self.sounds['police'].fadeout(60)
			self.sounds['police'].play( )

		#remove the cops who are not live
		self.cops = nlist

		#for updating the tree angles
		self.map.update( )

		
		#animate the woodgrowuplist
		nlist = []
		minsize = 0.2
		maxsize = .5
		for e in self.woodgrowup:
			e.ts += 0.04
			if e.ts <= 1.0:
				nlist.append( e )			
			e.y = float( 5*e.ts ) 
			e.size = float(minsize + e.ts * maxsize ) 
			e.opacity = (1.0-(e.ts))
		self.woodgrowup = nlist

	def drawRect( self, x1, y1, x2, y2, u1,v1, u2, v2 ):
		glBegin( GL_POLYGON )
		glTexCoord2f( u1, v1 )
		glVertex2f( x1, y1 )
		glTexCoord2f( u2, v1 )
		glVertex2f( x2, y1 )
		glTexCoord2f( u2, v2 )
		glVertex2f( x2, y2 )
		glTexCoord2f( u1, v2 )
		glVertex2f( x1, y2 )
		glEnd( )

	def drawHud( self ):
		glDisable( GL_DEPTH_TEST )
		glEnable( GL_ALPHA_TEST )
		glLoadIdentity( )
		glMatrixMode( GL_PROJECTION )
		glPushMatrix( )
		glLoadIdentity( )
		gluOrtho2D( 0, self.width, 0, self.height )
		
		#start drawing the hud
		glColor3f( 1, 1, 1 )
		if self.woodcount < 10 and self.gameWon==False:
			self.sounds['alarm'].play( )
			self.font.drawString( "LOW FUEL WARNING HARVEST WOOD", self.width/2.0-200, 50, 15, 15 )
		elif self.heat >= self.maxheat-10:
			self.font.drawString( "ENGINE HEATING REACH CHECKPOINT QUICKLY", self.width/2.0-150, 50, 15, 15 )
		else:
			if self.hero.objid == mymap.TYPE_TREE or self.hero.objid == mymap.TYPE_COCOTREE:
				self.font.drawString( "HOLD E TO CUT TREE", self.width/2.0-100, 50, 15, 15 )
			elif self.hero.objid == mymap.TYPE_TRAP:
				self.font.drawString( "U LOOSE WOOD IF YOU GO NEAR THE TRAP", self.width/2.0-150, 50, 15, 15 )
			elif self.hero.objid == mymap.TYPE_WALL or self.hero.objid == mymap.TYPE_WALL1:
				self.font.drawString( "USE W A S D TO MOVE", self.width/2.0-100, 50, 15, 15 )
			elif self.hero.objid == mymap.TYPE_DESTINATION and not(self.gameWon) and self.hero.alive :
				self.font.drawString( "ONE OR MORE OBJECTIVES NOT COMPLETED", self.width/2.0-200, 50, 15, 15 )

		if self.gameWon == True: 

			xmin = self.width/2.0-350
			xmax = self.width/2.0+350
			ymin = self.height-500
			ymax = self.height-250

			glEnable( GL_TEXTURE_2D )
			glEnable( GL_BLEND )
			glColor4f( 1, 1, 1, 0.8 )
			glBindTexture( GL_TEXTURE_2D, self.hud )
			self.drawRect( xmin, ymin, xmax, ymax, 0, 0.31, 1, 0.71 )
			glDisable( GL_BLEND )

			#screw
			self.drawRect( xmin+5, ymin+5, xmin+25, ymin+25, 0.257, 0, 0, 0.32 )
			self.drawRect( xmin+5, ymax-25, xmin+25, ymax-5, 0.257, 0, 0, 0.32 )
			self.drawRect( xmax-25, ymin+5, xmax-5, ymin+25, 0.257, 0, 0, 0.32 )
			self.drawRect( xmax-25, ymax-25, xmax-5, ymax-5, 0.257, 0, 0, 0.32 )
			
			glDisable( GL_TEXTURE_2D )

			glColor3f( 0, 1, 0 )
			self.font.drawString( "CHALLENGE COMPLETED", self.width/2.0-200, self.height-300, 20, 20 ) 
			glColor3f( 1, 1, 1 )
			self.font.drawString( "PRESS ENTER TO START NEXT LEVEL", self.width/2.0-300, self.height-350, 20, 20 )
			self.font.drawString( "PRESS ESC TO GO TO MENU", self.width/2.0-300, self.height-400, 20, 20 )

		elif self.hero.alive==False:

			xmin = self.width/2.0-350
			xmax = self.width/2.0+350
			ymin = self.height-500
			ymax = self.height-250

			glEnable( GL_TEXTURE_2D )
			glEnable( GL_BLEND )
			glColor4f( 1, 1, 1, 0.8 )
			glBindTexture( GL_TEXTURE_2D, self.hud )
			self.drawRect( xmin, ymin, xmax, ymax, 0, 0.31, 1, 0.71 )
			glDisable( GL_BLEND )

			#screw
			self.drawRect( xmin+5, ymin+5, xmin+25, ymin+25, 0.257, 0, 0, 0.32 )
			self.drawRect( xmin+5, ymax-25, xmin+25, ymax-5, 0.257, 0, 0, 0.32 )
			self.drawRect( xmax-25, ymin+5, xmax-5, ymin+25, 0.257, 0, 0, 0.32 )
			self.drawRect( xmax-25, ymax-25, xmax-5, ymax-5, 0.257, 0, 0, 0.32 )

			glColor3f( 1, 0, 0 )
			self.font.drawString( "CHALLENGE FAILED", self.width/2.0-200, self.height-300, 20, 20 ) 
			glColor3f( 1, 1, 1 )
			self.font.drawString( "PRESS ESC TO GO TO MENU", self.width/2.0-300, self.height-350, 20, 20 )

		if not(self.started):
			xmin = self.width/2.0-350
			xmax = self.width/2.0+350
			ymin = self.height-500
			ymax = self.height-250

			glEnable( GL_TEXTURE_2D )
			glEnable( GL_BLEND )
			glColor4f( 1, 1, 1, 0.8 )
			glBindTexture( GL_TEXTURE_2D, self.hud )
			self.drawRect( xmin, ymin, xmax, ymax, 0, 0.31, 1, 0.71 )
			glDisable( GL_BLEND )

			#screw
			self.drawRect( xmin+5, ymin+5, xmin+25, ymin+25, 0.257, 0, 0, 0.32 )
			self.drawRect( xmin+5, ymax-25, xmin+25, ymax-5, 0.257, 0, 0, 0.32 )
			self.drawRect( xmax-25, ymin+5, xmax-5, ymin+25, 0.257, 0, 0, 0.32 )
			self.drawRect( xmax-25, ymax-25, xmax-5, ymax-5, 0.257, 0, 0, 0.32 )

			glColor3f( 1, .80, .1 )
			self.font.drawString( "LEVEL: "+str(self.level+1), self.width/2.0-150, self.height-300, 20, 20 ) 
			self.font.drawString( "OBJECTIVES:", self.width/2.0-300, self.height-350, 20, 20 )
			glColor3f( 1, 1, 1 )
			self.font.drawString( "REACH DESTINATION BEFORE: WOOD IS ZERO OR ENGINE OVERHEATS", self.width/2.0-300, self.height-380, 10, 10 )
			if self.map.wood > 0:
				self.font.drawString( "COLLECT "+str(self.map.wood)+" WOOD", self.width/2.0-300, self.height-400, 10, 10 )
			if self.map.cops > 0:
				self.font.drawString( "TRAP "+str(self.map.cops)+" COPS", self.width/2.0-300, self.height-420, 10, 10 )
			glColor3f( 0, 1, 0 )
			self.font.drawString( "PRESS ENTER KEY TO CONTINUE", self.width/2.0-150, self.height-480, 10, 10 )



		h = self.height
		w = self.width
		hby4 = self.height/4.0
		wby4 = self.width/4.0

		#dimensions for temperature panel
		xmin = 0
		ymin = h-(hby4)+50
		xmax = 60
		ymax = h

		#caliculate heat val
#		percent = (self.maxheat-self.heat)/float(self.maxheat)
		percent = self.heat/float(self.maxheat)
		#temperature bar
#		glColor3f( percent, 1-percent, 0 )
		glColor3f( 1, 0, 0 )
		self.drawRect( xmin, ymin, xmax-5, ymin+(ymax-(ymin+10))*percent, 0.0, 0.3, 0.6, .8 )

		glEnable( GL_BLEND )
		glEnable( GL_TEXTURE_2D )

		#draw temperature bar
		glBindTexture( GL_TEXTURE_2D, self.heattex )
		glColor4f( 1, 1, 1, 1 )
		self.drawRect( xmin, ymin, xmax, ymax, 0.0, 0, 0.6, 1 )


		glColor4f( 1, 1, 1, 0.8 )
		#score panel
		glBindTexture( GL_TEXTURE_2D, self.hud )
		xmin = w-wby4
		ymin = h-(hby4)
		ymin = ymin+20
		xmax = w
		ymax = h
		self.drawRect( xmin, ymin, xmax, ymax, 0, 0.31, 1, 0.71 )
		glDisable( GL_BLEND )
		#screw
		self.drawRect( xmin+5, ymin+5, xmin+25, ymin+25, 0.257, 0, 0, 0.32 )
		self.drawRect( xmin+5, ymax-25, xmin+25, ymax-5, 0.257, 0, 0, 0.32 )
		self.drawRect( xmax-25, ymin+5, xmax-5, ymin+25, 0.257, 0, 0, 0.32 )
		self.drawRect( xmax-25, ymax-25, xmax-5, ymax-5, 0.257, 0, 0, 0.32 )
		#wood pic
		self.drawRect( xmin+20, ymax-50, xmin+60, ymax-10, 0.257, 0, 0.734, 0.31  )
		#cross mark for wood
		if self.woodcount < self.map.wood:
			self.drawRect( xmax-60, ymax-70, xmax-30, ymax-40, 0.46, 0.73, 0.765, 1 )
		else:
			self.drawRect( xmax-60, ymax-70, xmax-30, ymax-40, 0, 0.73, 0.46, 1 )
		#cop pic
		self.drawRect( xmin+20, ymax-120, xmin+60, ymax-80, 0.734, 0, 1, 0.31  )
		#cross mark for cop 
		if self.coptraps < self.map.cops:
			self.drawRect( xmax-60, ymax-140, xmax-30, ymax-110, 0.46, 0.73, 0.765, 1 )
		else:
			self.drawRect( xmax-60, ymax-140, xmax-30, ymax-110, 0, 0.73, 0.46, 1 )
		
		glDisable( GL_TEXTURE_2D )

		#Text
		glColor3f( 1, 1, 1 )
		self.font.drawString( "BLOW", 30, h-20, 10, 10 )
		self.font.drawString( "OK", 30, h-80, 10, 10 )
		self.font.drawString( "COOL", 30, h-120, 10, 10 )

		glColor3f( 1, .8, .1 )
		self.font.drawString( "TEMP", 0, h-5, 10, 10 )
		self.font.drawString( "WOOD:", xmin+80, ymax-20, 15, 15 )
		self.font.drawString( "TARGET:", xmin+60, ymax-50, 10, 10 )
		self.font.drawString( str(self.map.wood), xmin+150, ymax-50, 10, 10 )
	#	self.font.drawString( "CURRENT:", xmin+60, ymax-70, 10, 10 )
		#self.font.drawString( str(self.woodcount), xmin+150, ymax-70, 15, 15 )
		#wood count
		if self.woodcount < 10:
			glColor3f( 1, 0 , 0 )
		else:
			glColor3f( 1, 1, 1 )
		self.font.drawString( " "+str(self.woodcount), xmin+150, ymax-20, 15, 15 )

		glColor3f( 1, .8, .1 )
		self.font.drawString( "COPS:", xmin+80, ymax-90, 15, 15 )
		self.font.drawString( "TARGET:", xmin+60, ymax-120, 10, 10 )
		self.font.drawString( str(self.map.cops), xmin+150, ymax-120, 10, 10 )
#		self.font.drawString( "CURRENT:", xmin+60, ymax-140, 10, 10 )
		#self.font.drawString( str(self.coptraps), xmin+150, ymax-140, 10, 10 )
		
		glColor3f( 1, 1, 1 )
		#cop count
		self.font.drawString( " "+str(self.coptraps), xmin+150, ymax-90, 15, 15 )
	

		

		#end drawing hud
		glPopMatrix( )
		glEnable( GL_DEPTH_TEST )
		glDisable( GL_ALPHA_TEST )

	def display( self ):
		
		glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT )
		if not(self.menu.done):
			self.menu.render( )
			glFlush( )
			pygame.display.flip( )
			return 

		glMatrixMode( GL_PROJECTION )
		glLoadIdentity( )
		gluPerspective( 30, 1.33, 0.1, 400)
		glMatrixMode( GL_MODELVIEW )
		glLoadIdentity( )
		gluLookAt( self.camx, self.camy, self.camz, self.camx, 0, 20, 0, 1, 0 )	
				
		#drawing the ground
		glColor3f( 1, 1, 1 )
		glEnable( GL_TEXTURE_2D )
		glBindTexture( GL_TEXTURE_2D, self.ground )
		glBegin( GL_POLYGON )
		glTexCoord2f( 0, 0 )
		glVertex3f( -20, 0, -20 ) 
		glTexCoord2f( 8, 0 )
		glVertex3f( 80, 0, -20 )
		glTexCoord2f( 8, 6 )
		glVertex3f( 80, 0, 40 )
		glTexCoord2f( 0, 6 )
		glVertex3f( -20, 0, 40 )
		glEnd()
		glDisable( GL_TEXTURE_2D )

		#render the contents of map
		self.map.render( self.camx )
		
		#render the hero of game
		self.hero.render( )

		#render the cops in game
		cx = int( self.camx )
		for cop in self.cops:
			if cx-10 < int(cop.x) and int(cop.x) < cx+10:
				cop.render( )
		#draw the numbers
		for e in self.woodgrowup:
			glEnable( GL_BLEND )
			glEnable( GL_ALPHA_TEST )
			glEnable( GL_TEXTURE_2D )
			glDepthMask(0)
			glBindTexture( GL_TEXTURE_2D, e.texid )
			glColor4f( 1, 1, 1, e.opacity )
			glBegin(GL_QUADS);
			glTexCoord2f(0, 0); 
			glVertex3f(-1*e.size+e.x,-1*e.size+e.y, -0.+e.z);
			glTexCoord2f(0, 1); 
			glVertex3f(-1*e.size+e.x, 1*e.size+e.y,  0.+e.z);
			glTexCoord2f(1, 1);	
			glVertex3f( 1.*e.size+e.x, 1*e.size+e.y,  0.+e.z);
			glTexCoord2f(1, 0);	
			glVertex3f( 1.*e.size+e.x,-1.*e.size+e.y, -0.+e.z);
			glEnd();
			glDepthMask(1)
			glDisable( GL_ALPHA_TEST )
			glDisable( GL_BLEND )
		
		glPushMatrix()
		glColor3f( 1, 1, 1 );
		glTranslatef( self.hero.x, 1.5, self.hero.z )
		self.smoke.render( )
		glPopMatrix()

		self.drawHud( )

		glFlush( )
		pygame.display.flip( )

	def procEvents( self, event ):
		
		if not(self.menu.done):
			txt = self.menu.keyEvents( event )
			if txt == 'START':
				self.menu.done = True
				self.restart( )
				if self.soundsEnabled:
					pygame.mixer.music.load( os.path.join("music","background1.ogg") )
			elif txt == 'EXIT':
				sys.exit(0)
			return 
		
		if event.type == KEYDOWN:

			#user has to press ENTER TO START THE GAME
			if event.key == K_RETURN:
				self.started = True
				self.uy = 0
				self.camy = 6.2

			#disable other keys if game not started
			if self.started == False:
				return 

			#camera movement keys
#			if event.key == K_RIGHT:
#				self.ux = 0.5
#			elif event.key == K_LEFT:
#				self.ux = -0.5	
			if event.key == K_PAGEDOWN:
				self.uy = -0.5
			elif event.key == K_PAGEUP:
				self.uy = 0.5
				
			#player movement keys
			if event.key == K_w or event.key == K_UP :
			#	self.hero.moveTop( self.map )
				self.hero.direction = myobjects.TOP
			elif event.key == K_s or event.key == K_DOWN :
				#self.hero.moveBack( self.map )
				self.hero.direction = myobjects.BACK
			elif event.key == K_a or event.key == K_LEFT :
				#self.hero.moveLeft( self.map )
				self.hero.direction = myobjects.LEFT
			elif event.key == K_d or event.key == K_RIGHT :
				#self.hero.moveRight( self.map )
				self.hero.direction = myobjects.RIGHT

			elif event.key == K_ESCAPE:
				self.done = True
				self.menu.done = False
				if self.soundsEnabled:
					pygame.mixer.music.load( os.path.join("music",random.choice(["Grunge.wav","Rave.wav"])) )

			#cop trap key  
			elif event.key == K_SPACE:
				self.map.addTrap( int(self.hero.x), int(self.hero.z) )
				#play trap sounds
				self.sounds['trap'].play( )
				#reducing the wood and adding sprite
				sprite = mysmoke.elem()
				sprite.x = int(self.hero.x)
				sprite.y = .5
				sprite.z = int(self.hero.z)
				sprite.texid = self.growdowntex
				self.woodgrowup.append( sprite)
				self.woodcount -= 20

			elif event.key == K_r:
				self.restart( )

			elif event.key == K_e:
				#User has typed E for cutting the tree.
				if self.hero.objid == mymap.TYPE_TREE or self.hero.objid == mymap.TYPE_COCOTREE:
					x = int(self.hero.x)
					y = int(self.hero.z)
					self.map.startAnimation( )
					self.collecting = True


					#user started cutting Tree
					if self.hero.prevdir == myobjects.LEFT:
						self.map.curtree= [ x-1, y ] 
						#rotate about z
						self.map.setAxis([ 0, 0, 1 ])
						print "Tree is to Left"
					elif self.hero.prevdir == myobjects.RIGHT:
						self.map.curtree= [ x+1, y ] 
						#rotate about z
						self.map.setAxis( [ 0, 0, 1 ] )
						print "Tree is to Right"
					elif self.hero.prevdir == myobjects.TOP:
						self.map.curtree= [ x, y-1 ] 
						#rotate about x
						self.map.setAxis( [ 1, 0, 0 ] )
						print "Tree is to Top"
					elif self.hero.prevdir == myobjects.BACK:
						self.map.curtree= [ x, y+1 ] 
						#rotate about x
						self.map.setAxis( [ 1, 0, 0 ] )
						print "Tree is to Back"

			#special key to set follow
			if event.key == K_h:
				if self.follow == None:
					self.follow = self.hero
				else:
					self.follow = None
			#for starting next level
			if event.key == K_RETURN:
				if self.gameWon:
					self.level += 1
					self.level %= len(self.maps)
					self.mapname = os.path.join( "maps", self.maps[self.level]+".txt" )
					self.restart( )
			
		if event.type == self.TimerEvent:
			if self.hero.alive:
				self.update( )
			
		if event.type == KEYUP:
#			if event.key == K_RIGHT or event.key == K_LEFT:
#				self.ux = 0
			if event.key == K_PAGEUP or event.key == K_PAGEDOWN:
				self.uy = 0
			#hero keys
			if event.key == K_e:
				#stop animating tree	
				self.map.stopAnimation( )
				self.collecting = False

		if event.type is MOUSEMOTION:
			(x,y) =  event.pos
			if( x >= self.width - 10 ):
				self.ux = 0.5
			elif( x < 10 ):
				self.ux = -0.5
			else:
				self.ux = 0

		if event.type is MOUSEBUTTONDOWN:
			(x,y) = event.pos
			print x,y 

if __name__ == "__main__":
	pygame.init( )
	game = Game( )
	pygame.display.set_mode( (game.width, game.height), OPENGL|DOUBLEBUF)
	game.init( )
	while 1:
		event = pygame.event.poll( )
		if event.type is QUIT:
			sys.exit(0)
#		if not(game.done):
		game.display( )
		game.procEvents( event )

		if not(pygame.mixer.music.get_busy()) and game.soundsEnabled:
			pygame.mixer.music.rewind()
			pygame.mixer.music.play()
#		game.update( )
