from OpenGL.GL import *
from OpenGL.GLU import *
import pygame
from pygame import locals
import mytexture;



class Normal:
	def __init__( self ):
		self.x = 0;
		self.y = 0;
		self.z = 0;

class Vertex:
	def __init__( self , V=[0,0,0]):
		self.x = float(V[0]);
		self.y = float(V[1]);
		self.z = float(V[2]);
		self.normal = Normal( );

class Texture:
	"""Represents the texture coordinates
	in the AC3D object. """
	def __init__( self,l=[0,0] ):
		self.x = float(l[0]);
		self.y = float(l[1]);

class Surface:
	"""Represents the surfaces in the
		AC3D object """
	def __init__( self ):
		self.vertref= [];
		self.tex= [];
		self.normal = Normal( );
		self.mat = 0;

class ACObject:
	"""Represents the 3d Object 
	very minimalistic """
	def __init__( self ):
		self.loc = Normal( );	
		self.name = "";
		self.vertices = [];
		self.surfaces = [];
		self.kids = [];
		self.texture = -1;

	def animate( self ):
		pass;

	def render( self ):
		#print "Texture id",self.texture
		
		glEnable( GL_ALPHA_TEST )
		#vertices of each object
		verts = self.vertices;
		if self.texture != -1 : 
			glEnable( GL_TEXTURE_2D )
			glBindTexture( GL_TEXTURE_2D, self.texture );	
		glPushMatrix( );
		glTranslatef( self.loc.x, self.loc.y, self.loc.z );
		for surf in self.surfaces:
	  		#vertex refs for each surface
			vrefs = surf.vertref;
			if len(vrefs) == 3:
				glBegin( GL_TRIANGLES )
			else:
				glBegin( GL_POLYGON )
			for i in range( len(vrefs) ):
				glTexCoord2f( surf.tex[i].x, surf.tex[i].y )
				glVertex3f( verts[vrefs[i]].x, verts[vrefs[i]].y, verts[vrefs[i]].z  )
			glEnd( );
		for kid in self.kids:
			kid.render( );
		glPopMatrix( );
		glDisable( GL_TEXTURE_2D )
		glDisable( GL_ALPHA_TEST )

	def readSurf(self):
		self.fp.readline()
		surf=Surface()
		surf.mat=int(self.fp.readline().split()[1])
		numrefs = int(self.fp.readline().split( )[1]);
		for i in xrange(numrefs):
			l=self.fp.readline().split( );
			surf.vertref.append(int(l[0]))
			surf.tex.append(Texture(l[1:]))
		self.surfaces.append( surf );
	
	def readObject( self ):
		while 1:
			line=self.fp.readline()
			ftoken=line.split()[0]
#			print "Line:",line, "Token:", ftoken
			if ftoken == 'OBJECT':
				pass;
			if ftoken=='name':
				pass;
				
			elif ftoken=='crease':
				pass
				
			elif ftoken=='loc':
				l=line.split()
				self.loc.x=float(l[1]);
				self.loc.y=float(l[2]);
				self.loc.z=float(l[3]);
				
			elif ftoken=='texture':
				texfname = line.split( )[-1][1:-1];
				self.texture = mytexture.loadTexture(texfname)
				#print "Texture name: ", texfname
				#print self.texture, "TEXTURE ID"
    
			elif ftoken=='numvert':
				num = int(line.split()[1])
				for i in xrange(num):
					tmpvertex = Vertex( self.fp.readline( ).split( ) );
					self.vertices.append(tmpvertex)
					
			elif ftoken=='numsurf':
				num = int(line.split()[1])
				for i in xrange(num):
					self.readSurf( );
					
			elif ftoken=='kids':
				num = int(line.split()[1])
				for i in xrange( num ):
					tmp_obj = ACObject( );
					tmp_obj.fp = self.fp;
					tmp_obj.readObject( );
					self.kids.append( tmp_obj );
				return ;

	
	def load( self, objfname ):
		try:
			self.fp = file(objfname,"r");
			print objfname
			line=self.fp.readline();
			while line:
				line=self.fp.readline()
				ftoken=line.split()[0]
				if ftoken=='MATERIAL':
					pass
				elif ftoken=='OBJECT':
					self.readObject( );
		except :
			self.fp.close( );

