from OpenGL.GL import *
from OpenGL.GLU import *

from pygame.locals import *
import pygame

class Vertex:
	def __init__( self ):
		self.x = 0 
		self.y = 0
		self.z = 0

class MenuItem:
	def __init__(self, text, fnt ):
		self.text = text;
		#self.width = len(text)*10.0;
		self.width = 80;
		self.height = 15.0;
		self.center = Vertex( );
		self.active = 0;
		self.font = fnt;
		
	def isInside(self, x, y ):
		left = self.center.x - self.width/2.0;
		right = self.center.x + self.width/2.0;
		top = self.center.y - self.height/2.0;
		bottom = self.center.y + self.height/2.0;

		if( x < right and x > left ):
			if( y > bottom and y < top ):
				return 1;
			else: 
				return 0;
		else:
			return 0;
	def setCenter(self, x, y, z ):
		self.center.x = x;
		self.center.y = y;
		self.center.z = z;
		
	def setActive(self, value = 1):
		self.active = value;

	def render(self):
		left = self.center.x - self.width/2.0;
		right = self.center.x + self.width/2.0;
		top = self.center.y - self.height/2.0;
		bottom = self.center.y + self.height/2.0;
		
#		#print  left, right, top, bottom ;
		glDisable(GL_TEXTURE_2D);
		if(self.active):
			glColor4f( .5, .4, .1, .5 );
			glEnable( GL_ALPHA_TEST );
			glEnable( GL_BLEND )
			glBegin( GL_POLYGON );
			glVertex3f( left, top, self.center.z );
			glVertex3f( left, bottom, self.center.z );
			glVertex3f( right, bottom, self.center.z );
			glVertex3f( right, top, self.center.z );
			glEnd();
			glDisable( GL_BLEND )
			glDisable( GL_ALPHA_TEST );

		glPolygonMode( GL_FRONT_AND_BACK, GL_LINE );
		glColor4f( .5, .4, .1, 1 );
		glBegin( GL_POLYGON );
		glVertex3f( left, top, self.center.z );
		glVertex3f( left, bottom, self.center.z );
		glVertex3f( right, bottom, self.center.z );
		glVertex3f( right, top, self.center.z );
		glEnd();
		glPolygonMode( GL_FRONT_AND_BACK, GL_FILL );
		glEnable( GL_TEXTURE_2D );
		glEnable( GL_ALPHA_TEST )
		glColor3f( 1, .8, .1 )
		self.font.drawString( self.text, left, top+12,  10, 10 );
		glDisable( GL_ALPHA_TEST )



class MenuPage:
	def __init__(self, font, model, v=["None", "None", "None", "None"] ):
		self.items= [];
		self.font ="";
		self.curid = 0;
		self.done = 0;
		self.font = font; 
		self.model = model;
		self.mpageText = [ ];
		self.fontsize = 10;

		for i in xrange(len(v)):
			item = MenuItem (v[i],self.font);
			item.center.x = 0;
			item.center.y = 30-i*(item.height+10);
			item.center.z = 0;
			self.items.append(item); 

	def view2D( self ):
		#glDisable(GL_DEPTH_TEST);
		glDepthMask( 0 )
		glViewport( 0, 0, 1024, 768 );
		glMatrixMode( GL_PROJECTION );
		glLoadIdentity( );
		glOrtho( -100*1.33, 100*1.33, -100, 100, -100, 100 );
		glMatrixMode( GL_MODELVIEW );
		glLoadIdentity( )

		
	#render all the menu items in this menu page
	def render(self):
		#set the cur item active
		self.items[self.curid].setActive( );
		self.view2D( );
		glColor3f(1, 1, 1 );
		glEnable( GL_TEXTURE_2D )
		glBindTexture( GL_TEXTURE_2D, self.model );
		glBegin(GL_POLYGON);
		glTexCoord2f(0,0);glVertex3f( -133, -100, -10 );
		glTexCoord2f(1,0);glVertex3f( 133, -100, -10 );
		glTexCoord2f(1,1);glVertex3f( 133, 100, -10 );
		glTexCoord2f(0,1);glVertex3f( -133, 100, -10 );
		glEnd( );
		glDisable( GL_TEXTURE_2D )

		for i in xrange(len(self.mpageText)):
		    self.font.drawString( self.mpageText[i], -130, 80-i*self.fontsize, self.fontsize, self.fontsize );
		    
		for item in self.items:
			item.render( );
			item.setActive(0);
			
	def keyEvents(self, event):
		#manage the key input
		if event.type is KEYDOWN:
			if event.key == K_UP:
				self.curid = (self.curid-1)%len(self.items);		
			if event.key == K_DOWN:
				self.curid = (self.curid+1)%len(self.items);		
			if event.key == K_RETURN:
				#activate the menu item
				#self.done = 1;
				#if self.items[self.curid].text == 'EXIT':
				#	self.quit = 1;
				return self.items[self.curid].text;

		#manage mouse events
		elif event.type is MOUSEBUTTONDOWN:
			for i in xrange(len(self.items)):
				mx, my = pygame.mouse.get_pos( );
				if self.items[i].isInside( mx, my ) == 1:
					self.curid = i;	
